package com.softslate.commerce.administrator.core;

import java.io.File;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;
import com.softslate.commerce.daos.core.DAOUtils;
import com.softslate.commerce.daos.core.DAOUtilsImpl;

/**
 * 
 * @author David Tobey
 */
public class CreateKeysAction extends BaseAction {

	static Log log = LogFactory.getLog(CreateKeysAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		ActionErrors errors = new ActionErrors();
		ActionMessages messages = new ActionMessages();
		BaseForm baseForm = new BaseForm();
		baseForm.initializeProperties(mapping, request);
		boolean error = false;

		// Attempt to create the necessary key files on the server.
		DAOUtils daoUtils = new DAOUtilsImpl();
		String twoWayKeyFile = baseForm.getBusinessObjectFactory()
				.getAppSettings().getProperty("twoWayKeyFile");
		File keyFile = new File(twoWayKeyFile);
		if (keyFile.exists()) {
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.twoWayKeyFileExists"));
		} else {
			if (log.isDebugEnabled())
				log.debug("Attempting to create key file");
			try {
				if (twoWayKeyFile.length() > 0) {
					daoUtils.generateTwoWayKey(twoWayKeyFile);
				}
				messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"messages.twoWayKeyFileCreated"));
			} catch (Exception e) {
				log.error(e.toString(), e);
				errors.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
						"messages.keyFileError", e.toString()));
				error = true;
			}
		}

		if (!messages.isEmpty())
			addMessages(request, messages);
		if (!errors.isEmpty())
			addErrors(request, errors);
		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		if (error) {
			return mapping.findForward("failure");
		}
		return mapping.findForward("success");
	}
}
