package com.softslate.commerce.administrator.core;

import java.io.File;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.core.BaseForm;

/**
 * @author David Tobey
 */
public class UpgradesAction extends BaseAction {

	static Log log = LogFactory.getLog(UpgradesAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		BaseForm baseForm = new BaseForm();
		baseForm.setServlet(getServlet());
		baseForm.initializeProperties(mapping, request);

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("upgradesEdit")) {
			request.setAttribute("editPermission", "false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Set the current version of the database objects.
		String currentVersion = (String) baseForm.getSettings().getValue(
				"dataObjectsVersion");
		if (currentVersion == null) {
			currentVersion = "1.0.3";
		}

		// Set the database type - mssql or mysql.
		String dbType = baseForm.getBusinessObjectFactory().getDaoFactory()
				.getDatabaseType().toLowerCase();

		// Set the path to the upgrades directory.
		String pathToUpgrades = getServlet().getServletContext().getRealPath(
				"/WEB-INF/classes/resources/upgrades/");

		// Collect the upgrades into a sorted ArrayList
		ArrayList upgrades = collectUpgrades(currentVersion, pathToUpgrades,
				dbType);

		if (log.isDebugEnabled())
			log.debug("upgrades: " + upgrades);

		request.setAttribute("upgrades", upgrades);

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Upgrades" });
		request.setAttribute("breadcrumbs", bcs);

		if (log.isDebugEnabled())
			log.debug("Finished with Action.");
		return mapping.findForward("success");
	}

	public ArrayList collectUpgrades(String currentVersion,
			String pathToUpgrades, String dbType) {
		ArrayList upgrades = new ArrayList();
		File upgradesDirectory = new File(pathToUpgrades);
		if (!upgradesDirectory.isDirectory()) {
			return upgrades;
		}
		String[] files = upgradesDirectory.list();
		for (int i = 0; i < files.length; i++) {
			String regEx = "^upgrade_([^_]*)_([^_]*)_?([^\\.]*)\\.sql$";
			Pattern pattern = Pattern.compile(regEx, Pattern.CASE_INSENSITIVE);
			Matcher matcher = pattern.matcher(files[i]);
			if (matcher.find()) {
				String upgradeFromVersion = matcher.group(1);
				String upgradeToVersion = matcher.group(2);
				String databaseType = matcher.group(3);
				if (databaseType.equals("")) {
					databaseType = "all";
				}
				if (log.isDebugEnabled())
					log.debug("upgradeFromVersion: " + upgradeFromVersion);
				if (log.isDebugEnabled())
					log.debug("upgradeToVersion: " + upgradeToVersion);
				if (log.isDebugEnabled())
					log.debug("databaseType: " + databaseType);
				int lessEqualOrGreater = isVersionLessEqualOrGreater(currentVersion,
						upgradeFromVersion);
				if ((databaseType.equals("all") || databaseType.equals(dbType))
						&& (lessEqualOrGreater ==  -1 || lessEqualOrGreater == 0)) {
					Map upgrade = new HashMap();
					upgrade.put("upgradeFromVersion", upgradeFromVersion);
					upgrade.put("upgradeToVersion", upgradeToVersion);
					upgrade.put("databaseType", databaseType);
					addInOrder(upgrade, upgrades);
				}
			}
		}
		return upgrades;
	}

	public void addInOrder(Map upgrade, ArrayList upgrades) {
		if (upgrades.size() == 0) {
			upgrades.add(upgrade);
			return;
		}
		String upgradeFromVersion = (String) upgrade.get("upgradeFromVersion");
		Iterator i = upgrades.iterator();
		int count = 0;
		int position = 0;
		boolean positionFound = false;
		while (i.hasNext() && !positionFound) {
			Map thisUpgrade = (Map) i.next();
			String thisVersion = (String) thisUpgrade.get("upgradeFromVersion");
			int lessEqualOrGreater = isVersionLessEqualOrGreater(
					upgradeFromVersion, thisVersion);
			if (log.isDebugEnabled())
				log.debug("lessEqualOrGreater: " + lessEqualOrGreater);
			if (lessEqualOrGreater == 1) {
				position = count + 1;
			} else if (lessEqualOrGreater == 0) {
				// In case of a tie, the tie goes to the database-specific file
				boolean allDatabaseType = upgrade.get("databaseType") != null
						&& upgrade.get("databaseType").equals("all")
						&& thisUpgrade.get("databaseType") != null
						&& !thisUpgrade.get("databaseType").equals("all");
				if (allDatabaseType) {
					position = count + 1;
				} else {
					position = count;
					positionFound = true;
				}
			} else if (lessEqualOrGreater == -1) {
				position = count;
				positionFound = true;
			}
			count++;
		}
		if (log.isDebugEnabled())
			log.debug("position: " + position);
		if (log.isDebugEnabled())
			log.debug("upgradeFromVersion: " + upgradeFromVersion);
		upgrades.add(position, upgrade);
	}

	/*
	 * Return -1 if the given upgrade is less than, 0 if equal to, or 1 if
	 * greater than the one we're testing against.
	 */
	public int isVersionLessEqualOrGreater(String currentVersion,
			String testingVersion) {
		String[] currentVersionArray = currentVersion.split("\\.");
		String[] testingVersionArray = testingVersion.split("\\.");
		for (int i = 0; i < currentVersionArray.length; i++) {
			int current = Integer.parseInt(currentVersionArray[i]);
			if (testingVersionArray.length < i + 1) {
				return 1;
			}
			int testing = Integer.parseInt(testingVersionArray[i]);
			if (log.isDebugEnabled())
				log.debug("current: " + current);
			if (log.isDebugEnabled())
				log.debug("testing: " + testing);
			if (current < testing) {
				return -1;
			}
			if (current == testing && i == currentVersionArray.length - 1) {
				return 0;
			}
			if (current > testing) {
				return 1;
			}
		}
		return -1;
	}
}
