package com.softslate.commerce.administrator.email;

import java.util.Arrays;
import java.util.Collection;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.email.EmailOptOutProcessor;
import com.softslate.commerce.businessobjects.email.Template;
import com.softslate.commerce.businessobjects.email.TemplateProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * EmailOptOuts. Used for requests to "/administrator/EmailOptOut.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 */
public class EmailOptOutAction extends BaseAction {

	static Log log = LogFactory.getLog(EmailOptOutAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request, HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled()) log.debug("Starting action.");

		// Handle the main form for the control screen
		EmailOptOutForm baseForm = (EmailOptOutForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Handle the quick-add form
		EmailOptOutAddEditForm addForm = (EmailOptOutAddEditForm) request.getAttribute("emailOptOutAddEditForm");
		if (addForm == null || (request.getAttribute("reset") != null && request.getAttribute("reset").equals("true"))) {
			addForm = new EmailOptOutAddEditForm();
					}
		addForm.initializeProperties(mapping, request);
		request.setAttribute("emailOptOutAddEditForm", addForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request.getAttribute("businessObjectFactory");
		EmailOptOutProcessor processor =  bof.createObject(EmailOptOutProcessor.class);
		Map emailOptOutsAndCount = processor.loadEmailOptOutsAndCount(PropertyUtils.describe(baseForm));
		request.setAttribute("emailOptOuts", emailOptOutsAndCount.get("items"));
		request.setAttribute("itemCollection", emailOptOutsAndCount.get("items"));
		request.setAttribute("itemCount", emailOptOutsAndCount.get("itemCount"));
		
		TemplateProcessor tProcessor =  bof.createObject(TemplateProcessor.class);
		Collection<Template> templates = tProcessor.loadAllTemplates();
		request.setAttribute("templates", templates);

				// Enforce role permissions
		if (!request.isUserInRole("superuser") && !request.isUserInRole("administrator") && !request.isUserInRole("emailOptOutEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Create breadcrumbing collection
		Collection bcs = Arrays.asList(new String[] { "Home", "Email Opt Outs" });
		request.setAttribute("breadcrumbs", bcs);

				// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=emailOptOuts.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
