package com.softslate.commerce.administrator.product;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class that corresponds to requests to
 * "/administrator/CategoryDelete.do". Processes a request to delete records in
 * the system.
 * 
 * @author David Tobey
 */
public class CategoryDeleteAction extends CategoryEditAction {

    static Log log = LogFactory.getLog(CategoryDeleteAction.class);

    public ActionForward execute(ActionMapping mapping, ActionForm form,
            HttpServletRequest request, HttpServletResponse response)
            throws Exception {
        if (log.isDebugEnabled())
            log.debug("Starting action.");

        // Prepare parameters for the edit/delete processing, and process
        BusinessObjectFactory bof = (BusinessObjectFactory) request
                .getAttribute("businessObjectFactory");
        CategoryProcessor processor =  bof
                .createObject(CategoryProcessor.class);
        Map parameters = PropertyUtils.describe(form);
        processor.deleteCategory(parameters);

        // Reload category tree and builtin categories in the application scope.
        ProductInitializer pi = new ProductInitializer();
        pi.setServlet(servlet);
        pi.initializeCategoryTree(bof);
        pi.initializeBuiltInCategories(bof);

		// Set the success message
		CategoryAddEditForm baseForm = (CategoryAddEditForm) form;
		baseForm.initializeProperties(mapping, request);
		baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordDeleted"));

		// Optionally update the Lucene index
		updateAdvancedSearchIndex(baseForm, Integer.parseInt(baseForm.getCategoryID()));

		// Set the success message
		saveErrors(baseForm.getRequest(), baseForm.getErrors());
		saveMessages(request, baseForm.getMessages());
        return mapping.findForward("success");
    }
}
