package com.softslate.commerce.administrator.product;

import java.util.Map;
import java.util.HashMap;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.product.ProductInitializer;

/**
 * Struts action class for sorting category records alphabetically. Used for
 * requests to "/administrator/CategorySorter.do".
 * 
 * @author David Tobey
 */
public class CategorySorterAction extends BaseAction {

	static Log log = LogFactory.getLog(CategorySorterAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		// Sort the categories alphabetically by name or code
		ActionMessages messages = new ActionMessages();
		Map parameters = new HashMap();
		if (request.getParameter("parentCategory") != null
				&& request.getParameter("parentCategory").length() > 0
				&& !request.getParameter("parentCategory").equals("0")) {
			parameters.put("parentCategory", request
					.getParameter("parentCategory"));
		}
		parameters.put("workWithCategories", request
				.getParameter("workWithCategories"));

		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		CategoryProcessor processor =  bof
				.createObject(CategoryProcessor.class);
		Map results = null;
		if (request.getParameter("sortBy") != null
				&& request.getParameter("sortBy").equals("code")) {
			results = processor.sortCategoriesByCode(parameters);
			
			// Save a message
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.categoriesSortedByCode", results
							.get("categoriesSorted")));
			saveMessages(request, messages);
		} else {
			results = processor.sortCategoriesByName(parameters);
			
			// Save a message
			messages.add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage(
					"messages.categoriesSorted", results
							.get("categoriesSorted")));
			saveMessages(request, messages);
		}

		// Reload category tree and builtin categories in the application scope.
		ProductInitializer pi = new ProductInitializer();
		pi.setServlet(servlet);
		pi.initializeCategoryTree(bof);
		pi.initializeBuiltInCategories(bof);
		
		return mapping.findForward("success");
	}
}
