package com.softslate.commerce.administrator.product;

import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class corresponding to requests to
 * <code>/administrator/DiscountAddForm.do</code>. Creates an empty
 * <code>DiscountAddEditForm</code> object.
 * 
 * @author David Tobey
 */
public class DiscountAddFormAction extends BaseAction {

	static Log log = LogFactory.getLog(DiscountAddFormAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		DiscountAddEditForm baseForm = (DiscountAddEditForm) form;

		if (request.getAttribute("reset") != null
				&& request.getAttribute("reset").equals("true")) {
			Class classDefinition = baseForm.getClass();
			baseForm = (DiscountAddEditForm) classDefinition.newInstance();
		}

		baseForm.initializeProperties(mapping, request);
		request.setAttribute("discountAddEditForm", baseForm);
		baseForm.setProductID(request.getParameter("productID"));
		baseForm.setSkuID(request.getParameter("skuID"));

		// Load the parent product if an id is present
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load the parent sku if an id is present
		SKU sku = null;
		if (baseForm.getSkuID() != null && baseForm.getSkuID().length() > 0) {
			SKUProcessor skuProcessor =  bof
					.createObject(SKUProcessor.class);
			Map parameters = new HashMap();
			parameters.put("skuID", baseForm.getSkuID());
			sku = skuProcessor.loadSKUFromID(parameters);
			request.setAttribute("sku", sku);
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "Discounts", "Add Discount" });
			request.setAttribute("breadcrumbs", bcs);
		} else if (baseForm.getSkuID() != null
				&& baseForm.getSkuID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs",
					"SKU Details", "Discounts", "Add Discount" });
			request.setAttribute("breadcrumbs", bcs);
		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Discounts",
					"Add Discount" });
			request.setAttribute("breadcrumbs", bcs);
		}

		return mapping.findForward("success");
	}
}
