package com.softslate.commerce.administrator.product;

import java.awt.image.BufferedImage;
import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.Collection;
import java.util.Map;

import javax.imageio.ImageIO;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.lang3.StringEscapeUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.core.ApplicationMessage;
import com.softslate.commerce.businessobjects.product.CategoryProcessor;
import com.softslate.commerce.businessobjects.product.LuceneIndexer;
import com.softslate.commerce.businessobjects.product.ManufacturerProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.storage.FileStorageProcessor;
import com.softslate.commerce.customer.core.ActionUtils;
import com.softslate.commerce.customer.core.BaseAction;
import com.thebuzzmedia.imgscalr.Scalr;

/**
 * Struts action class that corresponds to requests to "/administrator/ProductEdit.do". Processes a request to edit a
 * record within the system.
 * 
 * @author David Tobey
 * 
 */
public class ProductEditAction extends BaseAction {

	static Log log = LogFactory.getLog(ProductEditAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form, HttpServletRequest request,
			HttpServletResponse response) throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");
		if (log.isInfoEnabled())
			log.info("Request params for multipart form data request: " + PropertyUtils.describe(form));
		ProductAddEditForm baseForm = (ProductAddEditForm) form;

		// Try uploading any files, these will throw exceptions if they do not
		// work therefore the product will not be added in that case.
		processImages(baseForm);

		// Handle generating the code from the name if it is empty
		if (baseForm.getProductID() == null
				&& ((baseForm.getCode() == null || baseForm.getCode().equals("")) && (baseForm.getName() != null && !baseForm
						.getName().equals("")))) {
			ActionUtils actionUtils = (ActionUtils) baseForm.getBusinessObjectFactory().createUtilsObject(
					"actionUtilsImplementer");
			baseForm.setCode(actionUtils.generateCodeFromName(baseForm.getName()));
		}

		// Handle setting the seoCode field to null if it is empty since it has a unique key
		Map parameters = baseForm.mapOfIncomingParams();
		if (parameters.get("seoCode") != null && parameters.get("seoCode").equals("")) {
			parameters.put("seoCode", null);
		}

		// Edit the record
		ProductProcessor processor = baseForm.getBusinessObjectFactory().createObject(ProductProcessor.class);
		Map results = processor.editProduct(parameters);

		// Load all categories for the primary category drop down.
		CategoryProcessor catProcessor = (CategoryProcessor) baseForm.getBusinessObjectFactory().createObject(
				"categoryProcessorImplementer");
		Collection categories = catProcessor.loadAllCategories();
		baseForm.getRequest().setAttribute("categories", categories);

		// Load all manufacturers for the manufacturer drop down.
		ManufacturerProcessor manfProcessor = (ManufacturerProcessor) baseForm.getBusinessObjectFactory().createObject(
				"manufacturerProcessorImplementer");
		Collection manufacturers = manfProcessor.loadAllManufacturers();
		request.setAttribute("manufacturerCollection", manufacturers);

		// Successful update
		if (results == null || results.get("resultCode") == null || results.get("resultCode").equals("0")) {

			// entity.jsp wants the updated record in the request scope
			Product product = (Product) results.get("updatedRecord");
			request.setAttribute("product", product);

			// Set the success messages
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("messages.recordSaved"));
			ActionUtils actionUtils = baseForm.getBusinessObjectFactory().createUtilsObject(ActionUtils.class);
			if (results != null) {
				actionUtils.processApplicationMessages(baseForm.getErrors(), baseForm.getMessages(),
						(Collection<ApplicationMessage>) results.get("applicationMessages"));
			}

			// Optionally update the Lucene index
			updateAdvancedSearchIndex(baseForm, product.getProductID());
		} else if (results != null && results.get("existingRecord") != null) {

			// Set the error messages
			Product existingRecord = (Product) results.get("existingRecord");
			String duplicateField = (String) results.get("duplicateField");
			String existingCode = existingRecord.getCode();
			if (duplicateField.equals("seoCode")) {
				existingCode = existingRecord.getSeoCode();
			}
			existingCode = StringEscapeUtils.escapeXml11(existingCode);
			request.setAttribute("okNotToEscapeMessages", Boolean.TRUE);
			baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.resultCode" + results.get("resultCode"), duplicateField, existingCode));
			baseForm.getErrors().add(
					ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.editExistingRecord", "ProductEditForm.do?productID="
							+ existingRecord.getProductID(), duplicateField, existingCode));
		}

		// Save errors and messages
		saveErrors(baseForm.getRequest(), baseForm.getErrors());
		saveMessages(baseForm.getRequest(), baseForm.getMessages());
		return mapping.findForward("success");
	}

	public void processImages(ProductAddEditForm productAddEditForm) throws Exception {

		// Try uploading any files, these will throw exceptions if they do not
		// work therefore the product will not be added in that case.
		ActionUtils actionUtils = (ActionUtils) productAddEditForm.getBusinessObjectFactory().createUtilsObject(
				"actionUtilsImplementer");
		FileStorageProcessor fsp = (FileStorageProcessor) productAddEditForm.getBusinessObjectFactory().createObject(
				"fileStorageProcessorImplementer");

		// Load the imageSystemPath and imagePathURL settings; derive the
		// imageSystemPath from the URL path if blank.
		String imageSystemPath = productAddEditForm.getSettings().getValue("imageSystemPath").toString();
		String imagePathURL = productAddEditForm.getSettings().getValue("imagePathURL").toString();
		if (!imagePathURL.endsWith("/")) {
			imagePathURL = imagePathURL + "/";
		}
		if (imageSystemPath == null || imageSystemPath.length() == 0) {
			imageSystemPath = getServlet().getServletContext().getRealPath(imagePathURL)
					+ System.getProperty("file.separator");
		}
		if (!imageSystemPath.endsWith(System.getProperty("file.separator"))) {
			imageSystemPath = imageSystemPath + System.getProperty("file.separator");
		}
		imagePathURL = productAddEditForm.getRequest().getContextPath() + imagePathURL;

		String automaticallyResizeImages = (String) productAddEditForm.getSettings().getValue(
				"automaticallyResizeImages");
		String smallImageSize = (String) productAddEditForm.getSettings().getValue("smallImageSize");
		String mediumImageSize = (String) productAddEditForm.getSettings().getValue("mediumImageSize");
		String largeImageSize = (String) productAddEditForm.getSettings().getValue("largeImageSize");

		if (productAddEditForm.getLargeImageUpload() != null
				&& productAddEditForm.getLargeImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(productAddEditForm.getLargeImageUpload().getFileName());
			if (automaticallyResizeImages != null && automaticallyResizeImages.equals("1")) {
				InputStream largeIS = null;
				InputStream smallIS = null;
				InputStream mediumIS = null;
				try {

					// Resize the large image if it's present and setting calls for it. If the setting is set to 0, do not resize, just store the raw, original image being uploaded.
					int largeImageSizeInt = Integer.parseInt(largeImageSize);
					largeIS = productAddEditForm.getLargeImageUpload().getInputStream();
					if (largeImageSizeInt > 0) {
						largeIS = resizeImage(productAddEditForm.getLargeImageUpload().getInputStream(),
								largeImageSizeInt);
					}
					String largefilename = "LG-" + filename;
					fsp.storeFile(largeIS, imageSystemPath + largefilename, "largeProduct");
					productAddEditForm.setLargeImage(fsp.urlForFile(imagePathURL, largefilename, "largeProduct"));

					// Automatically resize and create the small image, if it isn't present in the submission.
					if ((productAddEditForm.getSmallImage() == null || productAddEditForm.getSmallImage().length() == 0)
							&& (productAddEditForm.getSmallImageUpload() == null || productAddEditForm
									.getSmallImageUpload().getFileSize() == 0)) {
						int smallImageSizeInt = Integer.parseInt(smallImageSize);
						smallIS = productAddEditForm.getLargeImageUpload().getInputStream();
						if (smallImageSizeInt > 0) {
							smallIS = resizeImage(productAddEditForm.getLargeImageUpload().getInputStream(),
									smallImageSizeInt);
						}
						String smallfilename = "SM-" + filename;
						fsp.storeFile(smallIS, imageSystemPath + smallfilename, "smallProduct");
						productAddEditForm.setSmallImage(fsp.urlForFile(imagePathURL, smallfilename, "smallProduct"));
					}

					// Automatically resize and create the medium image, if it isn't present in the submission.
					if ((productAddEditForm.getMediumImage() == null || productAddEditForm.getMediumImage().length() == 0)
							&& (productAddEditForm.getMediumImageUpload() == null || productAddEditForm
									.getMediumImageUpload().getFileSize() == 0)) {
						int mediumImageSizeInt = Integer.parseInt(mediumImageSize);
						mediumIS = productAddEditForm.getLargeImageUpload().getInputStream();
						if (mediumImageSizeInt > 0) {
							mediumIS = resizeImage(productAddEditForm.getLargeImageUpload().getInputStream(),
									mediumImageSizeInt);
						}
						String mediumfilename = "MD-" + filename;
						fsp.storeFile(mediumIS, imageSystemPath + mediumfilename, "mediumProduct");
						productAddEditForm
								.setMediumImage(fsp.urlForFile(imagePathURL, mediumfilename, "mediumProduct"));
					}
				} finally {
					if (largeIS != null)
						largeIS.close();
					if (mediumIS != null)
						mediumIS.close();
					if (smallIS != null)
						smallIS.close();
				}
			} else {
				fsp.storeFile(productAddEditForm.getLargeImageUpload().getInputStream(), imageSystemPath + filename,
						"largeProduct");
				productAddEditForm.setLargeImage(fsp.urlForFile(imagePathURL, filename, "largeProduct"));
			}
		}

		// Process small and medium images if they are present in the
		// submission.
		if (productAddEditForm.getSmallImageUpload() != null
				&& productAddEditForm.getSmallImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(productAddEditForm.getSmallImageUpload().getFileName());
			fsp.storeFile(productAddEditForm.getSmallImageUpload().getInputStream(), imageSystemPath + filename,
					"smallProduct");
			productAddEditForm.setSmallImage(fsp.urlForFile(imagePathURL, filename, "smallProduct"));
		}
		if (productAddEditForm.getMediumImageUpload() != null
				&& productAddEditForm.getMediumImageUpload().getFileSize() > 0) {
			String filename = actionUtils.makeFilenameNice(productAddEditForm.getMediumImageUpload().getFileName());
			fsp.storeFile(productAddEditForm.getMediumImageUpload().getInputStream(), imageSystemPath + filename,
					"mediumProduct");
			productAddEditForm.setMediumImage(fsp.urlForFile(imagePathURL, filename, "mediumProduct"));
		}

	}

	private InputStream resizeImage(InputStream inputStream, int imageSize) throws IOException {
		BufferedImage srcImage = ImageIO.read(inputStream);
		BufferedImage scaledImage = Scalr.resize(srcImage, imageSize);
		ByteArrayOutputStream os = new ByteArrayOutputStream();
		ImageIO.write(scaledImage, "jpg", os);
		byte[] scaledImageContents = os.toByteArray();
		InputStream is = new ByteArrayInputStream(scaledImageContents);
		os.close();
		return is;
	}

	protected void updateAdvancedSearchIndex(ProductAddEditForm baseForm, int productID) {
		if (baseForm.getSettings().getValue("advancedSearchOn").equals("1") && baseForm.getUpdateAdvancedSearchIndex()) {
			Map indexerResults = null;
			try {
				LuceneIndexer indexer = baseForm.getBusinessObjectFactory().createObject(LuceneIndexer.class);
				String index = (String) baseForm.getSettings().getValue("luceneIndexDir");
				String indexDir = getServlet().getServletContext().getRealPath(index);
				indexerResults = indexer.addEditProduct(productID, indexDir);
				if (indexerResults.get("message") != null && ((String) indexerResults.get("message")).length() > 0) {
					baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
							new ActionMessage("errors.detail", indexerResults.get("message")));
				}
			} catch (Exception e) {
				log.error("Exception reindexing product: ", e);
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.detail", e.toString()));
			}
		}
	}
}
