package com.softslate.commerce.administrator.product;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Collection;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.beanutils.BeanUtils;
import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.product.AttributeProcessor;
import com.softslate.commerce.businessobjects.product.Discount;
import com.softslate.commerce.businessobjects.product.DiscountProcessor;
import com.softslate.commerce.businessobjects.product.Product;
import com.softslate.commerce.businessobjects.product.ProductProcessor;
import com.softslate.commerce.businessobjects.product.SKU;
import com.softslate.commerce.businessobjects.product.SKUProcessor;
import com.softslate.commerce.businessobjects.shipping.ShippingRule;
import com.softslate.commerce.businessobjects.shipping.ShippingRuleProcessor;
import com.softslate.commerce.customer.core.BaseAction;

/**
 * Struts action class that corresponds to the main administrator view panel for
 * skus. Used for requests to "/administrator/SKU.do".
 * <p>
 * The <code>execute</code> method of this class populates a
 * <code>Collection</code> named <code>itemCollection</code> in the request
 * scope containing the items to be displayed. Next, it sets a request scope
 * variable named <code>editPermission</code> that determines if the current
 * user has permission to edit the items. Finally, it creates a
 * <code>Collection</code> named <code>breadcrumbs</code> containing the
 * links and labels of each breadcrumb for display.
 * 
 * @author David Tobey
 */
public class SKUAction extends BaseAction {

	static Log log = LogFactory.getLog(SKUAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isDebugEnabled())
			log.debug("Starting action.");

		SKUForm baseForm = (SKUForm) form;
		baseForm.setProperties(request);
		request.setAttribute("mainForm", baseForm);

		// Get the collection of items to display
		BusinessObjectFactory bof = (BusinessObjectFactory) request
				.getAttribute("businessObjectFactory");
		SKUProcessor processor =  bof
				.createObject(SKUProcessor.class);
		Map skusAndCount = processor.loadSKUsAndCount(PropertyUtils
				.describe(baseForm));

		// Load the parent if an id is present
		Product product = null;
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {
			ProductProcessor productProcessor =  bof
					.createObject(ProductProcessor.class);
			Map parameters = new HashMap();
			parameters.put("productID", baseForm.getProductID());
			product = productProcessor.loadProductFromID(parameters);
			request.setAttribute("product", product);
		}

		// Load the parent if an id is present
		Discount discount = null;
		if (baseForm.getDiscountID() != null
				&& baseForm.getDiscountID().length() > 0) {
			DiscountProcessor discountProcessor =  bof
					.createObject(DiscountProcessor.class);
			Map parameters = new HashMap();
			parameters.put("discountID", baseForm.getDiscountID());
			discount = discountProcessor.loadDiscountFromID(parameters);
			request.setAttribute("discount", discount);
		}

		// Load the parent if an id is present
		ShippingRule shippingRule = null;
		if (baseForm.getShippingRuleID() != null
				&& baseForm.getShippingRuleID().length() > 0) {
			ShippingRuleProcessor shippingRuleProcessor =  bof
					.createObject(ShippingRuleProcessor.class);
			Map parameters = new HashMap();
			parameters.put("shippingRuleID", baseForm.getShippingRuleID());
			shippingRule = shippingRuleProcessor
					.loadShippingRuleFromID(parameters);
			request.setAttribute("shippingRule", shippingRule);
		}

		// Merge each id pair into the attributeOption properties.
		// And look up the Attributes and Options for each sku
		ProductProcessor productProcessor =  bof
				.createObject(ProductProcessor.class);
		AttributeProcessor attributeProcessor =  bof
				.createObject(AttributeProcessor.class);
		Collection items = (Collection) skusAndCount.get("items");
		Collection newItems = new ArrayList();
		Iterator i = items.iterator();
		while (i.hasNext()) {
			SKU sku = (SKU) i.next();
			SKUAddEditForm skuForm = new SKUAddEditForm();
			BeanUtils.copyProperties(skuForm, sku);
			skuForm.mergeAttributeOptions();
			Map item = PropertyUtils.describe(sku);
			item.put("attributeOption1", skuForm.getAttributeOption1());
			item.put("attributeOption2", skuForm.getAttributeOption2());
			item.put("attributeOption3", skuForm.getAttributeOption3());
			item.put("attributeOption4", skuForm.getAttributeOption4());
			item.put("attributeOption5", skuForm.getAttributeOption5());
			if (sku.getProductID() != null
					&& sku.getProductID().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("productID", sku.getProductID());
				item.put("product", productProcessor
						.loadProductFromID(parameters));
			}
			if (sku.getAttributeID1() != null
					&& sku.getAttributeID1().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("attributeID", sku.getAttributeID1());
				item.put("attribute1", attributeProcessor
						.loadAttributeFromID(parameters));
			}
			if (sku.getAttributeID2() != null
					&& sku.getAttributeID2().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("attributeID", sku.getAttributeID2());
				item.put("attribute2", attributeProcessor
						.loadAttributeFromID(parameters));
			}
			if (sku.getAttributeID3() != null
					&& sku.getAttributeID3().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("attributeID", sku.getAttributeID3());
				item.put("attribute3", attributeProcessor
						.loadAttributeFromID(parameters));
			}
			if (sku.getAttributeID4() != null
					&& sku.getAttributeID4().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("attributeID", sku.getAttributeID4());
				item.put("attribute4", attributeProcessor
						.loadAttributeFromID(parameters));
			}
			if (sku.getAttributeID5() != null
					&& sku.getAttributeID5().intValue() != 0) {
				Map parameters = new HashMap();
				parameters.put("attributeID", sku.getAttributeID5());
				item.put("attribute5", attributeProcessor
						.loadAttributeFromID(parameters));
			}
			newItems.add(item);
		}

		// Set request scope vars
		request.setAttribute("skus", newItems);
		request.setAttribute("itemCollection", newItems);

		// Problems with Gson serializing newItems, so if an API request, use items.
		if (request.getAttribute("isApiRequest") != null && (Boolean) request.getAttribute("isApiRequest")) {
			request.setAttribute("skus", items);
			request.setAttribute("itemCollection", items);
		}
		request.setAttribute("itemCount", skusAndCount.get("itemCount"));
		request
				.setAttribute("assignedItems", skusAndCount
						.get("assignedItems"));

		// Enforce role permissions
		if (!request.isUserInRole("superuser")
				&& !request.isUserInRole("administrator")
				&& !request.isUserInRole("skuEdit")) {
			request.setAttribute("editPermission", "false");
			baseForm.setEditMode("false");
		} else {
			request.setAttribute("editPermission", "true");
		}

		// Navigation
		if (baseForm.getProductID() != null
				&& baseForm.getProductID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Products",
					"Product Details", "SKUs" });
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getDiscountID() != null
				&& baseForm.getDiscountID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "Discounts",
					"Discount Details", "SKUs" });
			request.setAttribute("breadcrumbs", bcs);

		} else if (baseForm.getShippingRuleID() != null
				&& baseForm.getShippingRuleID().length() > 0) {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home",
					"Shipping Rules", "Shipping Rule Details", "SKUs" });
			request.setAttribute("breadcrumbs", bcs);

		} else {

			// Create breadcrumbing collection
			Collection bcs = Arrays.asList(new String[] { "Home", "SKUs" });
			request.setAttribute("breadcrumbs", bcs);
		}

		// Forward according to the outputMode of the request
		if (baseForm.getOutputMode().equals("print")) {
			request.setAttribute("outputMode", "print");
			baseForm.setOutputMode("screen");
			return mapping.findForward("successSimpleLayout");
		} else if (baseForm.getOutputMode().equals("excel")) {
			request.setAttribute("outputMode", "excel");
			baseForm.setOutputMode("screen");
			response.setHeader("Content-Type", "application/vnd.ms-excel");
			response.setHeader("Content-Disposition",
					"attachment; filename=skus.xls");
			return mapping.findForward("successSimpleLayout");
		}
		return mapping.findForward("success");
	}
}
