package com.softslate.commerce.businessobjects.core;

import java.util.Collection;
import java.util.Map;

import com.google.gson.ExclusionStrategy;
import com.google.gson.FieldAttributes;
import com.softslate.commerce.businessobjects.customer.Customer;

public class BasicJsonExclusionStrategy extends BaseBusinessProcessor implements ExclusionStrategy,
		JsonExclusionStrategy {

	public boolean shouldSkipField(FieldAttributes f) {
		boolean isFieldCollectionOrMap = isFieldCollectionOrMap(f);
		boolean isCustomerPasswordField = isCustomerPasswordField(f);
		boolean isCurrentAdministratorInRole = isCurrentAdministratorInRole(
				new String[] { "superuser", "customerPasswordEditor" });
		return isFieldCollectionOrMap || (isCustomerPasswordField && !isCurrentAdministratorInRole);
	}

	public boolean shouldSkipClass(Class<?> arg0) {
		return false;
	}

	protected boolean isCustomerPasswordField(FieldAttributes f) {
		boolean isCustomerPasswordField = false;
		Class clazz = f.getDeclaringClass();
		if (Customer.class.isAssignableFrom(clazz)) {
			String fieldName = f.getName();
			if (fieldName != null && (fieldName.equals("passwordToken") || fieldName.equals("passwordTokenDate")
					|| fieldName.equals("autoLoginToken") || fieldName.equals("decryptedPassword")
					|| fieldName.equals("password") || fieldName.equals("passwordExpires")
					|| fieldName.equals("numberOfFailedLogins") || fieldName.equals("lockedOutUntil")
					|| fieldName.equals("previousPasswords"))) {
				isCustomerPasswordField = true;
			}
		}
		return isCustomerPasswordField;
	}

	protected boolean isFieldCollectionOrMap(FieldAttributes f) {
		return Collection.class.equals(f.getDeclaredClass())
				|| (Map.class.equals(f.getDeclaredClass()) && !"jsonResult".equals(f.getName()));
	}

	protected boolean isCurrentAdministratorInRole(String[] roles) {
		boolean isCurrentAdministratorInRole = false;
		try {
			if (getUser() != null && getUser().getAdministrator() != null
					&& getUser().getAdministrator().getRoleCodes() != null) {
				for (int i = 0; i < roles.length; i++) {
					String role = roles[i];
					if (getUser().getAdministrator().getRoleCodes().contains(role)) {
						isCurrentAdministratorInRole = true;
						break;
					}
				}
			}
		} catch (Exception e) {
			log.error("Exception checking if administrator is in role (swalllowing): ", e);
		}
		return isCurrentAdministratorInRole;
	}
}
