package com.softslate.commerce.businessobjects.core;

import java.util.Map;

/**
 * A parent interface for all of the other interfaces in the business object
 * layer.
 * <p>
 * 
 * @author David Tobey
 * 
 * @see com.softslate.commerce.businessobjects.core.BaseBusinessObject
 */
public interface BusinessObject {

	/**
	 * Initializes this business object by copying incoming parameters into its
	 * properties. This method is used by
	 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
	 * immediately after object creation.
	 * 
	 * @param transferMap
	 *            A <code>Map</code> containing objects such as the
	 *            application's <code>DAOFactory</code>,
	 *            <code>Settings</code>,<code>User</code>, and
	 *            <code>BusinessObjectFactory</code>.
	 * @throws Exception
	 */
	abstract void initialize(Map transferMap) throws Exception;
}
