package com.softslate.commerce.businessobjects.customer;

import java.util.Collection;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a customer within the system. A customer is a user
 * that has established an account in the store with a user name and password.
 * Users who are not customers may still be able to place orders. The
 * "forceLogin" setting in the <code>appComponents.properties</code> fille table tells
 * the system to accept orders only from users who are customers and have logged
 * into their account.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Customer</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "customerImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "customerImplementer" is
 * {@link com.softslate.commerce.businessobjects.customer.CustomerBean}.
 * 
 * @author David Tobey
 * 
 * 
 */
@Audited
@AuditTable("AudCustomer")
public interface Customer extends BusinessObject {
	abstract int getCustomerID();

	abstract void setCustomerID(int customerID);

	abstract String getUserName();

	abstract void setUserName(String userName);

	abstract String getDecryptedPassword();

	abstract void setDecryptedPassword(String decryptedPassword);

	abstract String getPassword();

	abstract void setPassword(String password);

	abstract String getCreated();

	abstract void setCreated(String created);

	abstract String getLastLogin();

	abstract void setLastLogin(String lastLogin);

	abstract String getFormattedCreated();

	abstract void setFormattedCreated(String formattedCreated);

	abstract String getFormattedLastLogin();

	abstract void setFormattedLastLogin(String formattedLastLogin);

	abstract String getStatus();

	abstract void setStatus(String status);

	abstract String getFirstName();

	abstract void setFirstName(String firstName);

	abstract String getLastName();

	abstract void setLastName(String lastName);

	abstract String getOrganization();

	abstract void setOrganization(String organization);

	abstract String getAddress1();

	abstract void setAddress1(String address1);

	abstract String getAddress2();

	abstract void setAddress2(String address2);

	abstract String getCity();

	abstract void setCity(String city);

	abstract String getState();

	abstract void setState(String state);

	abstract String getCountry();

	abstract void setCountry(String country);

	abstract String getPostalCode();

	abstract void setPostalCode(String postalCode);

	abstract String getPhone1();

	abstract void setPhone1(String phone1);

	abstract String getPhone2();

	abstract void setPhone2(String phone2);

	abstract String getEmail1();

	abstract void setEmail1(String email1);

	abstract String getEmail2();

	abstract void setEmail2(String email2);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getExtra4();

	abstract void setExtra4(String extra4);

	abstract String getExtra5();

	abstract void setExtra5(String extra5);

	abstract String getNotes();

	abstract void setNotes(String notes);

	abstract String getAutoLoginToken();

	abstract void setAutoLoginToken(String autoLoginToken);

	abstract Collection getCustomerAddressCollection();

	abstract void setCustomerAddressCollection(
			Collection customerAddressCollection);

	abstract String getBillingFirstName();

	abstract void setBillingFirstName(String billingFirstName);

	abstract String getBillingLastName();

	abstract void setBillingLastName(String billingLastName);

	abstract String getBillingOrganization();

	abstract void setBillingOrganization(String billingOrganization);

	abstract String getBillingAddress1();

	abstract void setBillingAddress1(String billingAddress1);

	abstract String getBillingAddress2();

	abstract void setBillingAddress2(String billingAddress2);

	abstract String getBillingCity();

	abstract void setBillingCity(String billingCity);

	abstract String getBillingState();

	abstract void setBillingState(String billingState);

	abstract String getBillingCountry();

	abstract void setBillingCountry(String billingCountry);

	abstract String getBillingPostalCode();

	abstract void setBillingPostalCode(String billingPostalCode);

	abstract String getBillingPhone1();

	abstract void setBillingPhone1(String billingPhone1);

	abstract String getBillingPhone2();

	abstract void setBillingPhone2(String billingPhone2);

	abstract String getBillingEmail1();

	abstract void setBillingEmail1(String billingEmail1);

	abstract String getBillingEmail2();

	abstract void setBillingEmail2(String billingEmail2);

	abstract String getBillingExtra1();

	abstract void setBillingExtra1(String billingExtra1);

	abstract String getBillingExtra2();

	abstract void setBillingExtra2(String billingExtra2);

	abstract String getBillingExtra3();

	abstract void setBillingExtra3(String billingExtra3);

	abstract String getBillingExtra4();

	abstract void setBillingExtra4(String billingExtra4);

	abstract String getBillingExtra5();

	abstract void setBillingExtra5(String billingExtra5);

	abstract String getBillingNotes();

	abstract void setBillingNotes(String billingNotes);

	abstract CustomerAddress getFirstAddress();

	abstract Collection getOrderHistory();

	abstract void setOrderHistory(Collection orderHistory);

	@NotAudited
	abstract Collection getOrders();

	abstract void setOrders(Collection orders);

	abstract Collection getCustomerWishLists();

	abstract void setCustomerWishLists(Collection customerWishLists);

	abstract String getCimProfileID();

	abstract void setCimProfileID(String cimProfileID);

	abstract String getPasswordToken();

	abstract void setPasswordToken(String passwordToken);

	abstract String getPasswordTokenDate();

	abstract void setPasswordTokenDate(String passwordTokenDate);

	abstract boolean getCanUsePO();

	abstract void setCanUsePO(boolean canUsePO);

	abstract String getPasswordExpires();

	abstract void setPasswordExpires(String passwordExpires);

	abstract Integer getNumberOfFailedLogins();

	abstract void setNumberOfFailedLogins(Integer numberOfFailedLogins);

	abstract String getLockedOutUntil();

	abstract void setLockedOutUntil(String lockedOutUntil);

	abstract String getPreviousPasswords();

	abstract void setPreviousPasswords(String previousPasswords);
}
