package com.softslate.commerce.businessobjects.product;

import java.util.Collection;
import java.util.Map;

import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;
import org.hibernate.envers.NotAudited;

import com.softslate.commerce.businessobjects.core.BusinessObject;

/**
 * Interface representing a category within the system. A category is a
 * organization of products that have a common trait.
 * <p>
 * When the application needs to create an instance that implements
 * <code>Category</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "categoryImplementer" setting in the <code>appComponents.properties</code>
 * file.
 * <p>
 * The default "categoryImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.CategoryBean}.
 * 
 * @author David Tobey
 * 
 */
@Audited
@AuditTable("AudCategory")
public interface Category extends BusinessObject {
	abstract int getCategoryID();

	abstract void setCategoryID(int categoryID);

	abstract String getCode();

	abstract void setCode(String code);

	abstract String getName();

	abstract void setName(String name);

	abstract boolean getIsActive();

	abstract void setIsActive(boolean isActive);

	abstract int getCategoryTreeID();

	abstract void setCategoryTreeID(int categoryTreeID);

	abstract String getKeywords();

	abstract void setKeywords(String keywords);

	abstract String getExtra1();

	abstract void setExtra1(String extra1);

	abstract String getExtra2();

	abstract void setExtra2(String extra2);

	abstract String getExtra3();

	abstract void setExtra3(String extra3);

	abstract String getDescription();

	abstract void setDescription(String description);

	abstract String getHeader();

	abstract void setHeader(String header);

	abstract String getFooter();

	abstract void setFooter(String footer);

	abstract String getSmallImage();

	abstract void setSmallImage(String smallImage);

	abstract String getMediumImage();

	abstract void setMediumImage(String mediumImage);

	abstract String getLargeImage();

	abstract void setLargeImage(String largeImage);

	abstract Category getParent();

	abstract void setParent(Category parent);

	abstract Integer getParentCategory();

	abstract void setParentCategory(Integer parentCategory);

	abstract int getCategoryOrder();

	abstract void setCategoryOrder(int categoryOrder);

	abstract Collection getProductListCollection();

	abstract void setProductListCollection(Collection productListCollection);

	abstract Collection getParentCollection();

	abstract void setParentCollection(Collection parentCollection);

	abstract String getParentString();

	abstract void setParentString(String parentString);

	@NotAudited
	abstract Collection getSubcategoryCollection();

	abstract void setSubcategoryCollection(Collection subcategoryCollection);

	abstract int getItemsPerPage();

	abstract void setItemsPerPage(int itemsPerPage);

	abstract int getFirstRow();

	abstract void setFirstRow(int firstRow);

	abstract int getItemCount();

	abstract void setItemCount(int itemCount);

	abstract Map getCategorySettings();

	abstract void setCategorySettings(Map categorySettings);

	public Collection getProductCategories();

	public void setProductCategories(Collection productCategories);

	@NotAudited
	public Collection getProducts();

	public void setProducts(Collection products);

	public int getCategoryLevel();

	public void setCategoryLevel(int categoryLevel);

	public boolean getIsContentPage();

	public void setIsContentPage(boolean isContentPage);
}
