package com.softslate.commerce.businessobjects.product;

import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessProcessor;

/**
 * Interface for business logic methods having to do with discount ranges.
 * Created in the Struts layer in response to requests for viewing, editing,
 * adding, and deleting discount ranges.
 * <p>
 * When the application needs to create an instance that implements
 * <code>DiscountRangeProcessor</code>,
 * {@link com.softslate.commerce.businessobjects.core.BusinessObjectFactory}
 * finds the name of the Java class to instantiate from the
 * "discountRangeProcessorImplementer" setting in the
 * <code>appComponents.properties</code> file.
 * <p>
 * The default "discountRangeProcessorImplementer" is
 * {@link com.softslate.commerce.businessobjects.product.BasicDiscountRangeProcessor}.
 * 
 * @author David Tobey
 */
public interface DiscountRangeProcessor extends BusinessProcessor {

	abstract Map loadDiscountRangesAndCount(Map parameters) throws Exception;

	abstract Map addDiscountRange(Map parameters) throws Exception;

	abstract DiscountRange loadDiscountRangeFromID(Map parameters)
			throws Exception;

	abstract Map editDiscountRange(Map parameters) throws Exception;

	abstract Map editDeleteDiscountRanges(Map parameters) throws Exception;

	abstract void deleteDiscountRange(Map parameters) throws Exception;
}
