package com.softslate.commerce.businessobjects.shipping;

import java.io.Serializable;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.envers.AuditTable;
import org.hibernate.envers.Audited;

import com.softslate.commerce.businessobjects.core.BaseBusinessObject;
import com.softslate.commerce.businessobjects.product.SKU;

/**
 * Object representing a sku shipping rule in the system.
 * <code>SKUShippingRuleBean</code> is the default implementation of the
 * <code>SKUShippingRule</code> interface for the application.
 * <p>
 * The database stores sku shipping rules in the <code>sscSKUShippingRule</code> table,
 * and this interface can be used to represent a single row of that table. The
 * Hibernate configuration file <code>SKUShippingRuleBean.hbm.xml</code> maps that
 * table's columns to the properties in this object.
 * 
 * @author Jason McWilliams
 */
@Audited
@AuditTable("AudSKUShippingRule")
public class SKUShippingRuleBean extends BaseBusinessObject implements SKUShippingRule,
		Serializable {

	private static final long serialVersionUID = -7790026183450375818L;

	static Log log = LogFactory.getLog(SKUShippingRuleBean.class);

	private int skuShippingRuleID = 0;
	private int shippingRuleID = 0;
	private int skuID = 0;
	private ShippingRule shippingRule;
	private SKU sku;
	
	
	public int getSkuShippingRuleID() {
		return (this.skuShippingRuleID);
	}
	public void setSkuShippingRuleID(int skuShippingRuleID) {
		this.skuShippingRuleID = skuShippingRuleID;
	}

	public int getShippingRuleID() {
		return (this.shippingRuleID);
	}
	public void setShippingRuleID(int shippingRuleID) {
		this.shippingRuleID = shippingRuleID;
	}

	public int getSkuID() {
		return (this.skuID);
	}
	public void setSkuID(int skuID) {
		this.skuID = skuID;
	}

	public ShippingRule getShippingRule() {
		return shippingRule;
	}
	public void setShippingRule(ShippingRule shippingRule) {
		this.shippingRule = shippingRule;
	}

	public SKU getSku() {
		return sku;
	}
	public void setSku(SKU sku) {
		this.sku = sku;
	}
}
