package com.softslate.commerce.customer.core;

import java.util.Locale;
import java.util.Map;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.util.MessageResourcesFactory;
import org.apache.struts.util.PropertyMessageResources;

/**
 * Extension of the Struts <code>PropertyMessageResources</code> class which
 * wraps a second <code>PropertyMessageResources</code> object holding custom
 * application messages.
 * <p>
 * Messages that come distributed with the application are stored in the
 * <code>application.properties</code> files under the
 * "/WEB-INF/classes/resources" directory. These files may be overwritten with a
 * future upgrade, if new messages need to be added to the files. In order to
 * make use of the Struts <code>MessageResources</code> facility for custom
 * messages, this class automatically creates a second
 * <code>AppMessageResources</code> object from a file named
 * <code>application-custom.properties</code> in the same directory as the
 * distributed <code>application.properties</code> file. Custom messages can
 * safely be placed in these <code>application-custom.properties</code> files
 * because they will not be overwritten during an upgrade.
 * <p>
 * This class overrides a constructor method, to create the second, custom
 * <code>AppMessageResources</code> object from the
 * <code>application-custom.properties</code> file. It also overrides the
 * <code>getMessage</code> method to look for a message first in the custom
 * object before attempting to find it in the original
 * <code>AppMessageResources</code> object. This allows users to override the
 * default messages with custom ones.
 * 
 * @author David Tobey
 */
public class AppMessageResources extends PropertyMessageResources {

	private static final long serialVersionUID = -7020318529219376906L;

	static Log log = LogFactory.getLog(AppMessageResources.class);

	boolean isChild = false;

	AppMessageResources customMessageResources = null;

	public AppMessageResources getCustomMessageResources() {
		return customMessageResources;
	}

	public void setCustomMessageResources(
			AppMessageResources customMessageResources) {
		this.customMessageResources = customMessageResources;
	}

	public AppMessageResources(MessageResourcesFactory factory, String config,
			boolean returnNull, boolean isChild) {
		super(factory, config, returnNull);
		this.isChild = isChild;
		if (!isChild) {
			setCustomMessageResources(new AppMessageResources(factory, config
					+ "-custom", returnNull, true));
		}
	}

	public String getMessage(Locale locale, String key) {

		// If this is the child, custom object, just return the super's message
		if (isChild) {
			return super.getMessage(locale, key);
		}

		// Otherwise, look in the custom messages resource bundle first
		String customMessage = getCustomMessageResources().getMessage(locale,
				key);

		// Then, ask the super class
		if (customMessage == null
				|| (customMessage.startsWith("???") && customMessage
						.endsWith("???"))) {
			if (log.isDebugEnabled()) {
				log.debug("Custom message not found, "
						+ "looking for normal message (" + config + ","
						+ locale + "," + key + ")");
			}
			return super.getMessage(locale, key);
		}
		return customMessage;
	}
	
	public Map getMessages() {
		return messages;
	}
	
	public Map getLocales() {
		return locales;
	}
	
	public Map getFormats() {
		return formats;
	}
	
	public String messageKeyPublic(String locale, String key) {
        return super.messageKey(locale, key);
    }

}
