package com.softslate.commerce.customer.customer;

import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.businessobjects.customer.CustomerAddressProcessor;
import com.softslate.commerce.customer.core.BaseAction;
import com.softslate.commerce.customer.order.AddressBookAddressForm;

/**
 * 
 * @author Jason McWilliams
 * 
 */
public class EditAddressAction extends BaseAction {
	static Log log = LogFactory.getLog(EditAddressAction.class);

	public ActionForward execute(ActionMapping mapping, ActionForm form,
			HttpServletRequest request, HttpServletResponse response)
			throws Exception {
		if (log.isInfoEnabled())
			log.info("Starting action.");
		AddressBookAddressForm baseForm = (AddressBookAddressForm) form;
		synchronized (request.getSession()) {
			request.getSession().setAttribute("shippingAndTaxesProcessed",
					"false");

			// If the user is not logged in, it's an error.
			if (baseForm.getUser() == null
					|| baseForm.getUser().getCustomer() == null) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.userNotLoggedIn"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				return mapping.findForward("notLoggedIn");
			}
			String allowAddressBook = (String) baseForm.getSettings().getValue(
					"allowAddressBook");
			if (!"1".equals(allowAddressBook)) {
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage("errors.addressBookNotAllowed"));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Error: " + baseForm.getErrors().toString());
				return mapping.findForward("failure");
			}
			CustomerAddressProcessor customerAddressProcessor = (CustomerAddressProcessor) baseForm
					.getBusinessObjectFactory().createObject(
							"customerAddressProcessorImplementer");

			// Grab the specific fields we need as the form may be billing or
			// delivery
			Map addressParams;
			if (baseForm.getIsBilling()) {
				addressParams = baseForm.describeAddressType("billing");
			} else {
				addressParams = baseForm.describeAddressType("delivery");
			}
			if (addressParams != null) {
				// Add in the non-address fields for the update
				addressParams.put("customerAddressID", baseForm.getCustomerAddressID());
				addressParams.put("useForBilling", baseForm.getUseForBilling());
				addressParams.put("isBilling", baseForm.getIsBilling());
			}

			// Just to be sure, remove credit card info - we can't let users
			// update these directly
			addressParams.remove("paymentToken");
			addressParams.remove("creditCardName");
			addressParams.remove("creditCardType");
			addressParams.remove("creditCardDate");
			addressParams.remove("creditCardNumberDisplay");
			addressParams.remove("identifier");

			Map result = customerAddressProcessor.updateAddressBookAddress(addressParams);
			if (result != null && result.get("resultCode") != null
					&& !result.get("resultCode").equals("0")) {
				String resultCode = (String) result.get("resultCode");
				baseForm.getErrors().add(ActionMessages.GLOBAL_MESSAGE,
						new ActionMessage(resultCode));
				saveErrors(baseForm.getRequest(), (ActionMessages) baseForm
						.getErrors());
				if (log.isInfoEnabled() && !baseForm.getErrors().isEmpty())
					log.info("Validation error(s): "
							+ baseForm.getErrors().toString());
				baseForm.getRequest().setAttribute("editingAddressBook",
						baseForm.getCustomerAddressID());
				return mapping.findForward("failure");
			}
			
			// Set up the state and 'Other' fields for presentation.
			baseForm.prepareStateAndOtherFields(baseForm.getUser()
					.getCustomer());
					
			// Add a message to tell the user it worked
			baseForm.getMessages().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("messages.addressBookEdited"));
			saveMessages(baseForm.getRequest(), baseForm.getMessages());
			baseForm.getRequest().setAttribute("addressForm", baseForm);
			return mapping.findForward("success");
		}
	}
}
