package com.softslate.commerce.customer.customer;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class for submissions to log into a customer account. Used for
 * requests to "/Login.do".
 * <p>
 * The <code>validate</code> method of this class checks for the existance of
 * the <code>userName</code> and <code>decryptedPassword</code> properties.
 * 
 * @author David Tobey
 */
public class LoginForm extends BaseForm {
	private static final long serialVersionUID = 3616443510073668144L;

	static Log log = LogFactory.getLog(LoginForm.class);

	private String userName = null;

	public String getUserName() {
		return (this.userName);
	}

	public void setUserName(String userName) {
		this.userName = userName;
	}

	private String decryptedPassword = null;

	public String getDecryptedPassword() {
		return (this.decryptedPassword);
	}

	public void setDecryptedPassword(String decryptedPassword) {
		this.decryptedPassword = decryptedPassword;
	}

	private String rememberMe = null;

	public String getRememberMe() {
		return rememberMe;
	}

	public void setRememberMe(String rememberMe) {
		this.rememberMe = rememberMe;
	}

	private String originalRequest = null;

	public String getOriginalRequest() {
		return originalRequest;
	}

	public void setOriginalRequest(String originalRequest) {
		this.originalRequest = originalRequest;
	}

	private String csrfToken = null;

	public String getCsrfToken() {
		return csrfToken;
	}

	public void setCsrfToken(String csrfToken) {
		this.csrfToken = csrfToken;
	}

	public ActionErrors validate(ActionMapping mapping, HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		// Check for catalog mode
		try {
			String catalogMode = (String) getSettings().getValue("catalogMode");
			if (catalogMode.equals("1")) {
				getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.catalogMode"));
				return getErrors();
			}
		} catch (Exception e) {
			log.error(e.toString(), e);
		}
		if (getUserName() == null || getUserName().length() == 0) {
			String message = getMessage("customer", "customer.userName", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
		}
		if (getDecryptedPassword() == null || getDecryptedPassword().length() == 0) {
			String message = getMessage("customer", "customer.password", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.required", message));
		}
		// Ensure that the form submission is coming from the site itself
		if (getCsrfToken() == null || getCsrfToken().length() == 0
				|| !getCsrfToken().equals(getRequest().getSession().getAttribute("csrfToken"))) {
			getErrors().add(ActionMessages.GLOBAL_MESSAGE, new ActionMessage("errors.csrfError"));
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}
}
