package com.softslate.commerce.customer.payment;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;

import com.softslate.commerce.customer.core.ActionUtils;

/**
 * Struts form class for submissions of payment information for PayPal Payflow
 * Link during checkout. This form corresponds to the
 * {@link com.softslate.commerce.businessobjects.payment.PayflowLinkProcessor}
 * payment processor.
 * <p>
 * The database settings "activePaymentProcessors", "activePaymentForms", and
 * "activePaymentFormTemplate" represent the various payment methods available
 * to users. During checkout, each of the "activePaymentForms" will be
 * instantiated and prepared for display on the checkout payment screen. When
 * the user submits payment information the <code>validate</code> method of
 * the corresponding payment form is invoked.
 * <p>
 * The <code>validate</code> method of this class ensures that the incoming
 * credit card information is valid and complete. It then parses the database
 * settings for various PayflowLink fields and populates the corresponding form
 * properties.
 * 
 * @author David Tobey
 */
public class PayflowLinkForm extends BasicPaymentForm {

	private static final long serialVersionUID = 3257282539436914481L;

	static Log log = LogFactory.getLog(PayflowLinkForm.class);

	private String description = null;

	private String comment1 = null;

	private String comment2 = null;

	private String invoiceNumber = null;

	private String poNumber = null;

	public String getComment1() {
		return comment1;
	}

	public void setComment1(String comment1) {
		this.comment1 = comment1;
	}

	public String getComment2() {
		return comment2;
	}

	public void setComment2(String comment2) {
		this.comment2 = comment2;
	}

	public String getDescription() {
		return description;
	}

	public void setDescription(String description) {
		this.description = description;
	}

	public String getInvoiceNumber() {
		return invoiceNumber;
	}

	public void setInvoiceNumber(String invoiceNumber) {
		this.invoiceNumber = invoiceNumber;
	}

	public String getPoNumber() {
		return poNumber;
	}

	public void setPoNumber(String poNumber) {
		this.poNumber = poNumber;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		super.validate(mapping, request);
		ActionUtils actionUtils = (ActionUtils) getBusinessObjectFactory()
				.createUtilsObject("actionUtilsImplementer");
		// Populate description, comments, etc., parsing them for variables.
		try {
			setDescription(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowLinkDescription")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setComment1(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowLinkComment1")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setComment2(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowLinkComment2")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setInvoiceNumber(actionUtils
					.parseGenericTemplate(this, (String) getSettings()
							.getValue("payflowLinkInvoiceNumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		try {
			setPoNumber(actionUtils.parseGenericTemplate(this,
					(String) getSettings().getValue("payflowLinkPONumber")));
		} catch (Exception e) {
			if (log.isDebugEnabled())
				log.debug(e.toString());
		}
		if (log.isDebugEnabled()) {
			log.debug("description: " + getDescription());
			log.debug("comment1: " + getComment1());
			log.debug("comment2: " + getComment2());
			log.debug("invoice: " + getInvoiceNumber());
			log.debug("ponum: " + getPoNumber());
		}
		if (log.isDebugEnabled())
			log.debug("Finished validation.");
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());

		return getErrors();
	}

}
