package com.softslate.commerce.customer.product;

import javax.servlet.http.HttpServletRequest;

import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.apache.struts.action.ActionErrors;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.ActionMessage;
import org.apache.struts.action.ActionMessages;

import com.softslate.commerce.customer.core.BaseForm;

/**
 * Struts form class that correspondes to the search results page. Used for
 * requests to "/AdvancedSearch.do".
 * <p>
 * The <code>validate</code> method of this class makes sure that a
 * <code>searchString</code> of non-zero length was submitted, and sets the
 * directory where the index is to be found.
 * 
 * @author Jason McWilliams, David Tobey
 * 
 * @see com.softslate.commerce.customer.product.AdvancedSearchAction
 */
public class AdvancedSearchForm extends BaseForm {

	private static final long serialVersionUID = 691131667752592867L;

	static Log log = LogFactory.getLog(AdvancedSearchForm.class);

	String searchString = null;

	String indexDir = null;

	String firstRow = "1";

	String type = "1";

	String sortBy = "score";

	String categoryID = "0";

	String categoryCode = null;
	
	String ajax = "false";
	
	String[] facets = null;

	public String getSearchString() {
		return searchString;
	}

	public void setSearchString(String searchString) {
		this.searchString = searchString;
	}

	public String getIndexDir() {
		return indexDir;
	}

	public void setIndexDir(String indexDir) {
		this.indexDir = indexDir;
	}

	public String getCategoryID() {
		return categoryID;
	}

	public void setCategoryID(String categoryID) {
		this.categoryID = categoryID;
	}

	public String getCategoryCode() {
		return categoryCode;
	}

	public void setCategoryCode(String categoryCode) {
		this.categoryCode = categoryCode;
	}

	public String getFirstRow() {
		return firstRow;
	}

	public void setFirstRow(String firstRow) {
		this.firstRow = firstRow;
	}

	public String getSortBy() {
		return sortBy;
	}

	public void setSortBy(String sortBy) {
		this.sortBy = sortBy;
	}

	public String getType() {
		return type;
	}

	public void setType(String type) {
		this.type = type;
	}

	public String getAjax() {
		return ajax;
	}

	public void setAjax(String ajax) {
		this.ajax = ajax;
	}

	public String[] getFacets() {
		return facets;
	}

	public void setFacets(String[] facets) {
		this.facets = facets;
	}

	public ActionErrors validate(ActionMapping mapping,
			HttpServletRequest request) {
		if (log.isInfoEnabled())
			log.info("Starting validation.");
		initializeProperties(mapping, request);
		String index = (String) getSettings().getValue("luceneIndexDir");
		setIndexDir(getServlet().getServletContext().getRealPath(index));
		if (getSearchString() == null || getSearchString().length() == 0) {
			String message = getMessage("product", "search.fieldName", null);
			getErrors().add(ActionMessages.GLOBAL_MESSAGE,
					new ActionMessage("errors.required", message));
		}

		// Accommodate clients that submit the "facets" parameter as a comma-separated String rather than an array of
		// Strings. Eg., splits the String "root|category|YEARS,root|Planet|Mars" into a String array.
		if (getFacets() != null && getFacets().length == 1 && getFacets()[0].contains(",root")) {
			setFacets(getFacets()[0].split(","));
		}
		if (log.isInfoEnabled() && !getErrors().isEmpty())
			log.info("Validation error(s): " + getErrors().toString());
		return getErrors();
	}

}
