package com.softslate.commerce.daos.order;

import java.util.Collection;
import java.util.Map;

import com.softslate.commerce.businessobjects.core.BusinessObjectFactory;
import com.softslate.commerce.businessobjects.core.User;
import com.softslate.commerce.businessobjects.order.Order;
import com.softslate.commerce.businessobjects.order.OrderDelivery;
import com.softslate.commerce.businessobjects.order.OrderItem;
import com.softslate.commerce.businessobjects.order.OrderNumber;
import com.softslate.commerce.businessobjects.order.StatesAndCountries;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Interface for a DAO that runs miscellaneous queries related to orders.
 * 
 * @author David Tobey
 * 
 */
public interface OrderGatewayDAO extends DAOInterface {

	StatesAndCountries loadStatesAndCountries(
			StatesAndCountries statesAndCountries) throws Exception;

	Collection loadDashboardStatistics() throws Exception;

	Collection loadIncompleteOrdersStatistics() throws Exception;

	int deleteIncompleteOrders(boolean commit, Map parameters) throws Exception;

	void insertOrderAndDelivery(Order order, OrderDelivery orderDelivery)
			throws Exception;

	void processOrderItems(User user, Collection newOrderItems, Map parameters)
			throws Exception;

	void removeOrderItem(Order order, OrderDelivery orderDelivery,
			OrderItem orderItem) throws Exception;

	void updateOrderAndDelivery(boolean commit, Order order,
			OrderDelivery orderDelivery) throws Exception;

	Collection getProductInfoFromCodes(String[] productCodeArray)
			throws Exception;

	Map loadOrdersAndCount(Map parameters) throws Exception;

	Map editDeleteOrders(Map parameters, BusinessObjectFactory bof)
			throws Exception;

	void processOrderComplete(Order order, Map parameters) throws Exception;

	void insertOrderNumber(OrderNumber orderNumber, boolean commit)
			throws DataAccessException;

	void markOrderComplete(Order order, Map parameters) throws Exception;

	int deleteOrderDiscounts(boolean commit, int orderID) throws Exception;

	Collection getDiscountsBySkuIDs(Collection skuIDs) throws Exception;

	Collection getGlobalDiscounts() throws Exception;

	Collection getGlobalDiscounts(Collection couponCodes) throws Exception;

	Collection loadShippingDiscounts(Collection couponCodes) throws Exception;

	Collection loadGiftCertificates(Collection couponCodes) throws Exception;

	Double getPreviousOrderTotal(int customerID) throws Exception;

	Integer getTimesUsedByCustomer(int discountID, int customerID)
			throws Exception;

	Collection loadExport(String startDate, String endDate, boolean allOrders,
			int firstID, int lastID) throws Exception;

	Collection loadSalesByProductReport(String startDate, String endDate,
			Integer[] productIDs) throws Exception;

	Collection loadMonthlySalesReport(String startDate, String endDate)
			throws Exception;

	Collection loadIncompleteOrdersWithPayments(Map parameters)
			throws Exception;

	abstract int loadMaxOrderID() throws Exception;

	Collection<Order> loadCompletedOrdersByDateRange(String startDate, String endDate, Collection<String> ignoreStatuses)
			throws Exception;

	Collection<Order> loadAbandonedCartOrders(String startDate, String endDate, String prevOrdersDate) throws Exception;

	Collection<Order> loadCompletedOrdersByLastModifiedAndStatuses(String startDate, String endDate,
			Collection<String> currentStatuses) throws Exception;
}
