package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.OrderItemAttribute;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscOrderItemAttribute</code> table of the
 * database.
 * 
 * @author David Tobey
 * 
 */
public class OrderItemAttributeDAOHibernate extends BaseDAO implements
        DAOInterface, OrderItemAttributeDAO {

    static Log log = LogFactory.getLog(OrderItemAttributeDAOHibernate.class);

    private OrderItemAttribute orderItemAttribute = null;

    public OrderItemAttribute getOrderItemAttribute() {
        return (this.orderItemAttribute);
    }

    public void setOrderItemAttribute(OrderItemAttribute orderItemAttribute) {
        this.orderItemAttribute = orderItemAttribute;
    }

    public void loadOrderItemAttributeFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            OrderItemAttribute record = (OrderItemAttribute) session.get(
                    OrderItemAttribute.class, new Integer(
                            getOrderItemAttribute().getOrderItemAttributeID()));
            PropertyUtils.copyProperties(getOrderItemAttribute(), record);
            setOrderItemAttribute(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOrderItemAttribute() throws DataAccessException {
        insertOrderItemAttribute(false);
    }

    public void insertOrderItemAttribute(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getOrderItemAttribute());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOrderItemAttribute() throws DataAccessException {
        updateOrderItemAttribute(false);
    }

    public void updateOrderItemAttribute(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getOrderItemAttribute());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOrderItemAttribute() throws DataAccessException {
        deleteOrderItemAttribute(false);
    }

    public void deleteOrderItemAttribute(boolean commit)
            throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setOrderItemAttribute((OrderItemAttribute) session.load(
                    OrderItemAttribute.class, new Integer(
                            getOrderItemAttribute().getOrderItemAttributeID())));
            session.delete(getOrderItemAttribute());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
