package com.softslate.commerce.daos.order;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.order.OrderShippingRule;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscOrderShippingRule</code> table of the
 * database.
 * 
 * @author Jason McWilliams
 * 
 */
public class OrderShippingRuleDAOHibernate extends BaseDAO implements DAOInterface,
        OrderShippingRuleDAO {

    static Log log = LogFactory.getLog(OrderShippingRuleDAOHibernate.class);

    private OrderShippingRule orderShippingRule = null;

    public OrderShippingRule getOrderShippingRule() {
        return (this.orderShippingRule);
    }

    public void setOrderShippingRule(OrderShippingRule orderShippingRule) {
        this.orderShippingRule = orderShippingRule;
    }

    public void loadOrderShippingRuleFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            OrderShippingRule record = (OrderShippingRule) session.get(
                    OrderShippingRule.class, new Integer(getOrderShippingRule()
                            .getOrderShippingRuleID()));
            PropertyUtils.copyProperties(getOrderShippingRule(), record);
            setOrderShippingRule(record);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertOrderShippingRule() throws DataAccessException {
        insertOrderShippingRule(false);
    }

    public void insertOrderShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getOrderShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateOrderShippingRule() throws DataAccessException {
        updateOrderShippingRule(false);
    }

    public void updateOrderShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getOrderShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteOrderShippingRule() throws DataAccessException {
        deleteOrderShippingRule(false);
    }

    public void deleteOrderShippingRule(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setOrderShippingRule((OrderShippingRule) session.load(OrderShippingRule.class,
                    new Integer(getOrderShippingRule().getOrderShippingRuleID())));
            session.delete(getOrderShippingRule());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
