package com.softslate.commerce.daos.shipping;

import org.apache.commons.beanutils.PropertyUtils;
import org.apache.commons.logging.Log;
import org.apache.commons.logging.LogFactory;
import org.hibernate.HibernateException;
import org.hibernate.Session;

import com.softslate.commerce.businessobjects.shipping.ShippingRuleRange;
import com.softslate.commerce.daos.core.BaseDAO;
import com.softslate.commerce.daos.core.DAOInterface;
import com.softslate.commerce.daos.core.DataAccessException;

/**
 * Generic DAO to manage the <code>sscShippingRuleRange</code> table of the
 * database.
 * 
 * @author Jason McWilliams
 */
public class ShippingRuleRangeDAOHibernate extends BaseDAO implements DAOInterface,
        ShippingRuleRangeDAO {

    static Log log = LogFactory.getLog(ShippingRuleRangeDAOHibernate.class);

    private ShippingRuleRange shippingRuleRange = null;

    public ShippingRuleRange getShippingRuleRange() {
        return (this.shippingRuleRange);
    }

    public void setShippingRuleRange(ShippingRuleRange shippingRuleRange) {
        this.shippingRuleRange = shippingRuleRange;
    }

    public void loadShippingRuleRangeFromID() throws Exception {
        Session session = getDaoFactory().startSession();
        try {
            ShippingRuleRange shippingRuleRange = (ShippingRuleRange) session.get(
                    ShippingRuleRange.class, new Integer(getShippingRuleRange()
                            .getShippingRuleRangeID()));
            PropertyUtils.copyProperties(getShippingRuleRange(), shippingRuleRange);
            setShippingRuleRange(shippingRuleRange);
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void insertShippingRuleRange() throws DataAccessException {
        insertShippingRuleRange(false);
    }

    public void insertShippingRuleRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.save(getShippingRuleRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void updateShippingRuleRange() throws DataAccessException {
        updateShippingRuleRange(false);
    }

    public void updateShippingRuleRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            session.update(getShippingRuleRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }

    public void deleteShippingRuleRange() throws DataAccessException {
        deleteShippingRuleRange(false);
    }

    public void deleteShippingRuleRange(boolean commit) throws DataAccessException {
        Session session = getDaoFactory().startSession();
        try {
            setShippingRuleRange((ShippingRuleRange) session.load(ShippingRuleRange.class,
                    new Integer(getShippingRuleRange().getShippingRuleRangeID())));
            session.delete(getShippingRuleRange());
            if (commit) {
                getDaoFactory().commitTransaction();
            }
            evictCache();
        } catch (HibernateException he) {
            if (log.isErrorEnabled())
                log.error("Hibernate error: " + he.toString());
            throw new DataAccessException(he);
        }
        return;
    }
}
